package pro.litvinovg.w2phtml.gui;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.prefs.Preferences;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import static pro.litvinovg.w2phtml.gui.Contstants.*;


public class PreferencesStorage {
	
	private static final Logger logger = LoggerFactory.getLogger(PreferencesStorage.class);
	private Map<String,UIPreferences> tabs;

	private Preferences storage;

	public PreferencesStorage() {
		storage = Preferences.userRoot().node(this.getClass().getName());
		tabs = new HashMap();
		read();
		initializeDefault();
	}

	private void initializeDefault() {
		tabs.put(HTML, getDefaultHTML(this));
		tabs.put(EPUB,getDefaultEpub(this));
		tabs.put(RDF,getDefaultRDF(this));
	}

	public static UIPreferences getDefaultHTML(PreferencesStorage store) {
		UIPreferences prefs = new UIPreferences(store);
		prefs.put(PREF_RESOLUTIONS, new String[]{"0","72","150","300","600","1200","2400"} );
		prefs.put(PREF_DEFAULT_IMAGE_RESOLUTION,"2");
		prefs.put(PREF_USE_MATHJAX,"true");
		prefs.put(PREF_IGNORE_EMPTY_PARS,"false");
		prefs.put(PREF_IGNORE_HARD_BREAKS,"false");
		prefs.put(PREF_GREENSTONE_TAGS,"false");
		prefs.put(PREF_LETTER_SPACING,"0.15");
		prefs.put(PREF_INLINE_STYLES,"true");
		prefs.put(PREF_EMBED_RASTER,"true");
		prefs.put(PREF_EMBED_VECTOR,"true");
		prefs.put(PREF_CONVERT_TO_PX,"false");
		prefs.put(PREF_PAGEBREAK_STYLES,DEFAULT_PAGEBREAK_STYLE_SETTINGS);
		prefs.put(PREF_PAGINATION,"true");
		prefs.put(PREF_SPLIT_LEVELS, new String[]{"0","1","2","3","4","5","6","7","8","9","10"} );
		prefs.put(PREF_SPLIT_BY_LEVEL,"0");
		prefs.put(PREF_TYPE,HTML);
		prefs.put(PREF_MAX_WIDTH,"");
		prefs.put(PREF_SPLIT_WHOLE_PAGES,"false");
		prefs.put(PREF_SCALING, SCALING_DEFAULT_VALUE);
		prefs.setDefault(true);
		return prefs;
	}
	

	public static  UIPreferences getDefaultEpub(PreferencesStorage store) {
		UIPreferences prefs = new UIPreferences(store);
		prefs.put(PREF_RESOLUTIONS, new String[]{"0","72","150","300","600","1200","2400"});
		prefs.put(PREF_DEFAULT_IMAGE_RESOLUTION,"2");
		prefs.put(PREF_USE_MATHJAX,"true");
		prefs.put(PREF_IGNORE_EMPTY_PARS,"false");
		prefs.put(PREF_IGNORE_HARD_BREAKS,"false");
		prefs.put(PREF_LETTER_SPACING,"0.15");
		prefs.put(PREF_INLINE_STYLES,"true");
		prefs.put(PREF_EMBED_RASTER,"true");
		prefs.put(PREF_EMBED_VECTOR,"true");
		prefs.put(PREF_CONVERT_TO_PX,"false");
		prefs.put(PREF_SCALING, SCALING_DEFAULT_VALUE);
		prefs.put(PREF_PAGEBREAK_STYLES,DEFAULT_PAGEBREAK_STYLE_SETTINGS);
		prefs.put(PREF_PAGINATION,"true");
		prefs.put(PREF_TYPE,EPUB);
		prefs.setDefault(true);
		return prefs;
	}

	public static UIPreferences getDefaultRDF(PreferencesStorage store) {
		UIPreferences prefs = new UIPreferences(store);
		prefs.put(PREF_PUBLICATION_TYPES, new String[]{"elenphArticle", "encArticle", "book", "journal"} );
		prefs.put(PREF_RESOLUTIONS, new String[]{"0","72","150","300","600","1200","2400"} );
		prefs.put(PREF_DEFAULT_IMAGE_RESOLUTION,"2");
		prefs.put(PREF_DEFAULT_PUBLICATION_TYPE,"0");
		prefs.put(PREF_USE_MATHJAX,"true");
		prefs.put(PREF_IGNORE_EMPTY_PARS,"false");
		prefs.put(PREF_IGNORE_HARD_BREAKS,"false");
		prefs.put(PREF_LETTER_SPACING,"0.15");
		prefs.put(PREF_INLINE_STYLES,"true");
		prefs.put(PREF_EMBED_RASTER,"true");
		prefs.put(PREF_EMBED_VECTOR,"true");
		prefs.put(PREF_CONVERT_TO_PX,"false");
		prefs.put(PREF_PAGEBREAK_STYLES,DEFAULT_PAGEBREAK_STYLE_SETTINGS);
		prefs.put(PREF_TYPE,RDF);
		prefs.put(PREF_SCALING, SCALING_DEFAULT_VALUE);
		prefs.put(PREF_PAGINATION,"true");
		prefs.setDefault(true);
		return prefs;
	}

	public void put(String prefName, UIPreferences newPreferences) {
		tabs.put(prefName, newPreferences);
	}
	public boolean has(String configName) {
		return tabs.containsKey(configName);
	}
	
	private void add(String key, String[] strings,UIPreferences prefs) {
		prefs.put(key, strings);
	}
	
	private void add(String key, String value,UIPreferences prefs) {
		prefs.put(key, new String[] {value});
	}

	public UIPreferences get(String name) {
		if (tabs.containsKey(name)) {
			return tabs.get(name);	
		} else {
			logger.error("Prefereneces with name " + name + " not found.");
			return null;
		}
		
	}
	public void remove(String name) {
			 tabs.remove(name);	
	}
	
	public void setSourceFileName(String fileName) {
		Iterator<Entry<String, UIPreferences>> it = tabs.entrySet().iterator();
		while (it.hasNext()) {
			Entry<String, UIPreferences> pair = it.next();
			UIPreferences value = (UIPreferences) pair.getValue();
			value.setSourceFileName(fileName);
		}
	}

	public void save() {
		String jsonPrefereneces = convertAllToJSON();
		storage.put(TAB_PREFERENCES, jsonPrefereneces);
	}

	private String convertAllToJSON() {
		JSONObject json = new JSONObject();
		for (Map.Entry<String, UIPreferences> entry : tabs.entrySet()) {
			String name = entry.getKey();
			UIPreferences prefs = entry.getValue();
			JSONObject json_prefs = prefs.getJSON();
			json.put(name, json_prefs);
		}
		return json.toString();
	}
	
	public Set<String> getTabNames() {
		return tabs.keySet();
	}

	private void read() {
		String data = storage.get(TAB_PREFERENCES, null);
		if (data == null) {
			return;
		}
		try {
			JSONObject json = new JSONObject(data);
			Set<String> prefTabNames = json.keySet();
			for (String prefTabName : prefTabNames) {
				UIPreferences newPref = new UIPreferences(this);
				JSONObject tabData = (JSONObject) json.get(prefTabName);
				Set<String> prefKeys = tabData.keySet();
				for (String name : prefKeys) {
					JSONArray values = (JSONArray) tabData.get(name);
					if (values != null) {
						String[] valueArray = new String[values.length()];
					  for (int i=0;i<values.length();i++){ 
					   valueArray[i] = values.getString(i);
					  }
					  newPref.put(name, valueArray);
					} 
				}
				tabs.put(prefTabName, newPref);
			}
		} catch (JSONException e) {
			logger.error("JSON parsing exception " + e.getLocalizedMessage());
			e.printStackTrace();
		} catch (Exception e) {
			logger.error("JSON parsing exception " + e.getLocalizedMessage());
			e.printStackTrace();
		}
	}
}
	

